
#include <plant/Inter__plant__noChange.hpp>
#include <plant/ConnT__plant__noChange.hpp>

const bitset<3> Inter__plant__noChange::predefined(string("111"));

Inter__plant__noChange::Inter__plant__noChange(const ConnT__plant__noChange &connector) : Interaction(connector) {
    involvedPorts.reset();
    port_vector_fresh = true;
    defined = false;
    refresh_defined = false;
    mPorts.reserve(3);
}

Inter__plant__noChange::Inter__plant__noChange(const ConnT__plant__noChange &connector, bool p1, bool p2, bool p3) : Interaction(connector) {
    involvedPorts.reset();

    defined = true;
    refresh_defined = false;
	port_vector_fresh = false;

    involvedPorts.set(0, p1);
    involvedPorts.set(1, p2);
    involvedPorts.set(2, p3);

    defined = p1 && p2 && p3;
    mPorts.reserve(3);
}

Inter__plant__noChange::Inter__plant__noChange(const ConnT__plant__noChange &connector, const vector<Port *> &ports) : Interaction(connector), port_vector_fresh(false) {
    commonRecycle(ports);
    mPorts.reserve(3);
}

Inter__plant__noChange::~Inter__plant__noChange(){
}

// This one is 'const' but its only role
// is to set mPorts correctly (ie. its only role is to modify the object)
void Inter__plant__noChange::refreshPorts() const {
    const vector<QuotedPortReference *> &parent_ports = this->connector().ports();
    mPorts.clear();
    mPorts.reserve(3);

    if (involvedPorts.test(0)) {
         mPorts.push_back(&(parent_ports[0]->port()));
    }
    if (involvedPorts.test(1)) {
         mPorts.push_back(&(parent_ports[1]->port()));
    }
    if (involvedPorts.test(2)) {
         mPorts.push_back(&(parent_ports[2]->port()));
    }
    port_vector_fresh = true;
}

vector<Port*>& Inter__plant__noChange::ports() {
    if (!port_vector_fresh){
        refreshPorts();
    }

    return mPorts;
}

const vector<Port *> & Inter__plant__noChange::ports() const {
    if (!port_vector_fresh){
        refreshPorts();
    }

    return mPorts;
}

bool Inter__plant__noChange::operator==(const Interaction &interaction) const {
    bool ret = false;

    // check if interaction values are from the same connector
    if (&connector() == &interaction.connector()) {
        assert(dynamic_cast<const Inter__plant__noChange *>(&interaction) != NULL);
        const Inter__plant__noChange *other = static_cast<const Inter__plant__noChange *>(&interaction);
        ret = (involvedPorts == other->involvedPorts);
    }
    return ret;
}

bool Inter__plant__noChange::operator<=(const Interaction &interaction) const {
    bool ret = false;

    // check if interaction values are from the same connector
    if (&connector() == &interaction.connector()) {
        assert(dynamic_cast<const Inter__plant__noChange *>(&interaction) != NULL);
        const Inter__plant__noChange *other = static_cast<const Inter__plant__noChange *>(&interaction);
        ret = ((involvedPorts & other->involvedPorts) == involvedPorts);
    }

    return ret;
}

bool Inter__plant__noChange::operator<(const Interaction &interaction) const {
    bool ret = false;

    // check if interaction values are from the same connector
    if (&connector() == &interaction.connector()) {
        assert(dynamic_cast<const Inter__plant__noChange *>(&interaction) != NULL);
        const Inter__plant__noChange *other = static_cast<const Inter__plant__noChange *>(&interaction);
        ret = (((involvedPorts &  other->involvedPorts) == involvedPorts) &&
                (involvedPorts != other->involvedPorts));
    }

    return ret;
}

void Inter__plant__noChange::recycle() {
    Interaction::recycle();
    mPorts.clear();
    port_vector_fresh = true;
    defined = false;
    refresh_defined = false;
    involvedPorts.reset();
}

void Inter__plant__noChange::recycle(const vector<Port *> &ports) {
    Interaction::recycle();
    commonRecycle(ports);
}

void Inter__plant__noChange::commonRecycle(const vector<Port *> &ports){
    involvedPorts.reset();

    for (vector<Port *>::const_iterator portIt = ports.begin() ;
         portIt != ports.end() ;
         ++portIt) {
        const vector<Port *>::size_type post_shift = findPort(*portIt);

        involvedPorts.set(post_shift);
    }

    port_vector_fresh = false;
    refresh_defined = true;
}
